#!/bin/sh

xgettext --language="C#" --keyword=_ --escape --sort-output --from-code=utf-8 --output=deepin-terminal.pot ../main.vala ../widget/about_widget.vala ../widget/add_button.vala ../widget/workspace_manager.vala ../widget/remote_server_dialog.vala ../widget/confirm_dialog.vala ../widget/preference_slidebar.vala ../widget/preference.vala ../widget/terminal.vala ../widget/search_entry.vala ../widget/config_window.vala ../widget/appbar.vala ../widget/remote_panel.vala ../widget/tabbar.vala ../widget/file_button.vala ../widget/command_panel.vala ../widget/command_button.vala ../widget/command_dialog.vala ../widget/entry_menu.vala ../widget/shortcut_entry.vala
